@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 ## --------------------------------------------------------------------------
-##
-##   Copyright 1996-2020 The NASM Authors - All Rights Reserved
+##   
+##   Copyright 1996-2014 The NASM Authors - All Rights Reserved
 ##   See the file AUTHORS included with the NASM distribution for
 ##   the specific copyright holders.
 ##
@@ -15,7 +15,7 @@
 ##     copyright notice, this list of conditions and the following
 ##     disclaimer in the documentation and/or other materials provided
 ##     with the distribution.
-##
+##     
 ##     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 ##     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 ##     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
@@ -40,7 +40,7 @@
 
 require 'phash.ph';
 
-my($output, $insnsn_c, $regs_dat, $tokens_dat) = @ARGV;
+my($output, $insns_dat, $regs_dat, $tokens_dat) = @ARGV;
 
 %tokens = ();
 @tokendata = ();
@@ -53,18 +53,30 @@ my($output, $insnsn_c, $regs_dat, $tokens_dat) = @ARGV;
 	       'nle', 'no', 'np', 'ns', 'nz', 'o', 'p', 'pe', 'po', 's', 'z');
 
 #
-# Read insnsn.c
+# Read insns.dat
 #
-open(ID, '<', $insnsn_c) or die "$0: cannot open $insnsn_c: $!\n";
+open(ID, '<', $insns_dat) or die "$0: cannot open $insns_dat: $!\n";
 while (defined($line = <ID>)) {
-    next unless ($line =~ /^\s*\"([\?\@a-z0-9_]+)\"/);
+    if ($line =~ /^([A-Z0-9_]+)(|cc)\s/) {
+	$insn = $1.$2;
+	($token = $1) =~ tr/A-Z/a-z/;
 
-    my $token = $1;
-    next if (defined($tokens{$token}));	# This should never happen
-
-    $tokens{$token} = scalar @tokendata;
-    push(@tokendata, "\"${token}\", ".length($token).
-	 ", TOKEN_INSN, 0, 0, I_\U${token}");
+	if ($2 eq '') {
+	    # Single instruction token
+	    if (!defined($tokens{$token})) {
+		$tokens{$token} = scalar @tokendata;
+		push(@tokendata, "\"${token}\", TOKEN_INSN, C_none, 0, I_${insn}");
+	    }
+	} else {
+	    # Conditional instruction
+	    foreach $cc (@conditions) {
+		if (!defined($tokens{$token.$cc})) {
+		    $tokens{$token.$cc} = scalar @tokendata;
+		    push(@tokendata, "\"${token}${cc}\", TOKEN_INSN, C_\U$cc\E, 0, I_${insn}");
+		}
+	    }
+	}
+    }
 }
 close(ID);
 
@@ -73,7 +85,7 @@ close(ID);
 #
 open(RD, '<', $regs_dat) or die "$0: cannot open $regs_dat: $!\n";
 while (defined($line = <RD>)) {
-    if ($line =~ /^([\?\@a-z0-9_-]+)\s*\S+\s*\S+\s*[0-9]+\s*(\S*)/) {
+    if ($line =~ /^([a-z0-9_-]+)\s*\S+\s*\S+\s*[0-9]+\s*(\S*)/) {
 	$reg = $1;
 	$reg_flag = $2;
 
@@ -85,8 +97,7 @@ while (defined($line = <RD>)) {
 	    $reg_suffix = $4;
 	} else {
 	    $nregs = 1;
-	    undef $reg_prefix;
-	    undef $reg_suffix;
+	    undef $reg_prefix, $reg_suffix;
 	}
 
 	while ($nregs--) {
@@ -94,8 +105,11 @@ while (defined($line = <RD>)) {
 		die "Duplicate definition: $reg\n";
 	    }
 	    $tokens{$reg} = scalar @tokendata;
-	    $reg_flag = '0' if ($reg_flag eq '');
-	    push(@tokendata, "\"${reg}\", ".length($reg).", TOKEN_REG, 0, ${reg_flag}, R_\U${reg}\E");
+	    if ($reg_flag eq '') {
+	        push(@tokendata, "\"${reg}\", TOKEN_REG, 0, 0, R_\U${reg}\E");
+	    } else {
+	        push(@tokendata, "\"${reg}\", TOKEN_REG, 0, ${reg_flag}, R_\U${reg}\E");
+	    }
 
 	    if (defined($reg_prefix)) {
 		$reg_nr++;
@@ -114,10 +128,9 @@ close(RD);
 #
 open(TD, '<', $tokens_dat) or die "$0: cannot open $tokens_dat: $!\n";
 while (defined($line = <TD>)) {
-    $line =~ s/\s*(|\#.*)$//;
     if ($line =~ /^\%\s+(.*)$/) {
 	$pattern = $1;
-    } elsif ($line =~ /^(\S+)/) {
+    } elsif ($line =~ /^([a-z0-9_-]+)/) {
 	$token = $1;
 
 	if (defined($tokens{$token})) {
@@ -130,7 +143,6 @@ while (defined($line = <TD>)) {
 	    my $head = $1, $tail = $3;
 	    my $px = $2;
 
-	    $px =~ s/\?/\\?/g;
 	    $px =~ s/\*/(.*)/g;
 	    if ($token =~ /$px/i) {
 		$data = $head."\U$1".$tail;
@@ -140,25 +152,24 @@ while (defined($line = <TD>)) {
 	}
 
 	$data =~ s/\*/\U$token/g;
-	$data =~ s/\?//g;
 
-	push(@tokendata, "\"$token\", ".length($token).", $data");
+	push(@tokendata, "\"$token\", $data");
     }
 }
 close(TD);
 
-$max_len = 0;
-foreach $token (keys(%tokens)) {
-    if (length($token) > $max_len) {
-	$max_len = length($token);
-    }
-}
-
 if ($output eq 'h') {
     #
     # tokens.h
     #
 
+    $max_len = 0;
+    foreach $token (keys(%tokens)) {
+	if (length($token) > $max_len) {
+	    $max_len = length($token);
+	}
+    }
+
     print "/*\n";
     print " * This file is generated from insns.dat, regs.dat and token.dat\n";
     print " * by tokhash.pl; do not edit.\n";
@@ -185,8 +196,9 @@ if ($output eq 'h') {
     verify_hash_table(\%tokens, \@hashinfo);
 
     ($n, $sv, $g) = @hashinfo;
+    $sv2 = $sv+2;
+
     die if ($n & ($n-1));
-    $n <<= 1;
 
     print "/*\n";
     print " * This file is generated from insns.dat, regs.dat and token.dat\n";
@@ -195,6 +207,7 @@ if ($output eq 'h') {
     print "\n";
 
     print "#include \"compiler.h\"\n";
+    print "#include <string.h>\n";
     print "#include \"nasm.h\"\n";
     print "#include \"hashtbl.h\"\n";
     print "#include \"insns.h\"\n";
@@ -206,10 +219,9 @@ if ($output eq 'h') {
     # 64-bit machines and 12 bytes on 32-bit machines.
     print "struct tokendata {\n";
     print "    const char *string;\n";
-    print "    uint16_t len;\n";
     print "    int16_t tokentype;\n";
-    print "    int16_t aux;\n";
-    print "    uint16_t tokflag;\n";
+    print "    int8_t aux;\n";
+    print "    int8_t tokflag;\n";
     print "    int32_t num;\n";
     print "};\n";
     print "\n";
@@ -220,63 +232,53 @@ if ($output eq 'h') {
     # Put a large value in unused slots.  This makes it extremely unlikely
     # that any combination that involves unused slot will pass the range test.
     # This speeds up rejection of unrecognized tokens, i.e. identifiers.
-    print "#define INVALID_HASH_ENTRY (65535/3)\n";
+    print "#define UNUSED (65535/3)\n";
 
-    printf "    static const int16_t hashdata[%d] = {\n", $n;
+    print "    static const int16_t hash1[$n] = {\n";
     for ($i = 0; $i < $n; $i++) {
-	my $h = ${$g}[$i];
-	print "        ", defined($h) ? $h : 'INVALID_HASH_ENTRY', ",\n";
+	my $h = ${$g}[$i*2+0];
+	print "        ", defined($h) ? $h : 'UNUSED', ",\n";
     }
     print "    };\n";
 
-    printf "    static const struct tokendata tokendata[%d] = {\n",
-	scalar(@tokendata);
+    print "    static const int16_t hash2[$n] = {\n";
+    for ($i = 0; $i < $n; $i++) {
+	my $h = ${$g}[$i*2+1];
+	print "        ", defined($h) ? $h : 'UNUSED', ",\n";
+    }
+    print "    };\n";
+
+    printf "    static const struct tokendata tokendata[%d] = {\n", scalar(@tokendata);
     foreach $d (@tokendata) {
 	print "        { ", $d, " },\n";
     }
     print  "    };\n";
 
     print  "    uint32_t k1, k2;\n";
+    print  "    uint64_t crc;\n";
     # For correct overflow behavior, "ix" should be unsigned of the same
     # width as the hash arrays.
     print  "    uint16_t ix;\n";
     print  "    const struct tokendata *data;\n";
-    printf "    char lcbuf[%d];\n", $max_len+1;
-    print  "    const char *p = token;\n";
-    print  "    char c, *q = lcbuf;\n";
-    print  "    size_t len = 0;\n";
-    printf "    uint64_t crc = UINT64_C(0x%08x%08x);\n", $$sv[0], $$sv[1];
     print  "\n";
-    print  "    while ((c = *p++)) {\n";
-    printf "        if (++len > %d)\n", $max_len;
-    print  "            goto notfound;\n";
-    print  "        *q++ = c = nasm_tolower(c);\n";
-    print  "        crc = crc64_byte(crc, c);\n";
-    print  "    };\n";
+    printf "    tv->t_flag = 0;\n";
+    printf "    crc = crc64(UINT64_C(0x%08x%08x), token);\n",
+	$$sv[0], $$sv[1];
+    print  "    k1 = (uint32_t)crc;\n";
+    print  "    k2 = (uint32_t)(crc >> 32);\n";
     print  "\n";
-    printf "    k1 = ((uint32_t)crc & 0x%x) + 0;\n", $n-2;
-    printf "    k2 = ((uint32_t)(crc >> 32) & 0x%x) + 1;\n", $n-2;
-    print  "\n";
-    printf "    ix = hashdata[k1] + hashdata[k2];\n",
-	$n-2, $n-2;
+    printf "    ix = hash1[k1 & 0x%x] + hash2[k2 & 0x%x];\n", $n-1, $n-1;
     printf "    if (ix >= %d)\n", scalar(@tokendata);
-    print  "        goto notfound;\n";
+    print  "        return tv->t_type = TOKEN_ID;\n";
     print  "\n";
     print  "    data = &tokendata[ix];\n";
-    print  "    if (data->len != len)\n";
-    print  "        goto notfound;\n";
-    print  "    if (memcmp(data->string, lcbuf, len))\n";
-    print  "        goto notfound;\n";
+
+    print  "    if (strcmp(data->string, token))\n";
+    print  "        return tv->t_type = TOKEN_ID;\n";
     print  "\n";
     print  "    tv->t_integer = data->num;\n";
     print  "    tv->t_inttwo  = data->aux;\n";
     print  "    tv->t_flag    = data->tokflag;\n";
     print  "    return tv->t_type = data->tokentype;\n";
-    print  "\n";
-    print  "notfound:\n";
-    print  "    tv->t_integer = 0;\n";
-    print  "    tv->t_inttwo  = 0;\n";
-    print  "    tv->t_flag    = 0;\n";
-    print  "    return tv->t_type = TOKEN_ID;\n";
     print  "}\n";
 }
