@@ -44,9 +44,9 @@
  * An error reporting function should look like this.
  */
 void printf_func(2, 3) nasm_error(int severity, const char *fmt, ...);
-no_return printf_func(2, 3) nasm_fatal(int flags, const char *fmt, ...);
-no_return printf_func(2, 3) nasm_panic(int flags, const char *fmt, ...);
-no_return nasm_panic_from_macro(const char *file, int line);
+fatal_func printf_func(2, 3) nasm_fatal(int flags, const char *fmt, ...);
+fatal_func printf_func(2, 3) nasm_panic(int flags, const char *fmt, ...);
+fatal_func nasm_panic_from_macro(const char *file, int line);
 #define panic() nasm_panic_from_macro(__FILE__, __LINE__);
 
 typedef void (*vefunc) (int severity, const char *fmt, va_list ap);
@@ -71,13 +71,14 @@ static inline vefunc nasm_set_verror(vefunc ve)
                                          * and dump core for reference */
 #define ERR_MASK        0x00000007      /* mask off the above codes */
 #define ERR_NOFILE      0x00000010      /* don't give source file name/line */
-#define ERR_USAGE       0x00000020      /* print a usage message */
-#define ERR_PASS1       0x00000040      /* only print this error on pass one */
-#define ERR_PASS2       0x00000080
+#define ERR_TOPFILE	0x00000020      /* give the top input file name only */
+#define ERR_USAGE       0x00000040      /* print a usage message */
+#define ERR_PASS1       0x00000080      /* only print this error on pass one */
+#define ERR_PASS2       0x00000100      /* only print this error on pass one */
 
-#define ERR_NO_SEVERITY 0x00000100      /* suppress printing severity */
-#define ERR_PP_PRECOND	0x00000200	/* for preprocessor use */
-#define ERR_PP_LISTMACRO 0x00000400	/* from preproc->error_list_macros() */
+#define ERR_NO_SEVERITY 0x00000200      /* suppress printing severity */
+#define ERR_PP_PRECOND	0x00000400	/* for preprocessor use */
+#define ERR_PP_LISTMACRO 0x00000800	/* from preproc->error_list_macros() */
 
 /*
  * These codes define specific types of suppressible warning.
@@ -111,9 +112,11 @@ static inline vefunc nasm_set_verror(vefunc ve)
 #define ERR_WARN_UNKNOWN_PRAGMA	WARN(18) /* unknown pragma */
 #define ERR_WARN_NOTMY_PRAGMA	WARN(19) /* pragma inapplicable */
 #define ERR_WARN_UNK_WARNING	WARN(20) /* unknown warning */
+#define ERR_WARN_NEG_REP	WARN(21) /* negative repeat count */
+#define ERR_WARN_PHASE		WARN(22) /* phase error in pass 1 */
 
 /* The "all" warning acts as a global switch, it must come last */
-#define ERR_WARN_ALL            21 /* Do not use WARN() here */
+#define ERR_WARN_ALL            23 /* Do not use WARN() here */
 
 struct warning {
     const char *name;
