@@ -209,6 +209,7 @@ static void ieee_unqualified_name(char *, char *);
  */
 static void ieee_init(void)
 {
+    strlcpy(ieee_infile, inname, sizeof(ieee_infile));
     any_segs = false;
     fpubhead = NULL;
     fpubtail = &fpubhead;
@@ -807,10 +808,9 @@ static int32_t ieee_segment(char *name, int pass, int *bits)
 
         ieee_seg_needs_update = seg;
         if (seg->align >= SEG_ABS)
-            define_label(name, NO_SEG, seg->align - SEG_ABS,
-			 NULL, false, false);
+            define_label(name, NO_SEG, seg->align - SEG_ABS, false);
         else
-            define_label(name, seg->index + 1, 0L, NULL, false, false);
+            define_label(name, seg->index + 1, 0L, false);
         ieee_seg_needs_update = NULL;
 
         if (seg->use32)
@@ -886,15 +886,6 @@ static int32_t ieee_segbase(int32_t segment)
     return segment;             /* no special treatment */
 }
 
-/*
- * filename
- */
-static void ieee_filename(char *inname, char *outname)
-{
-    strcpy(ieee_infile, inname);
-    standard_extension(inname, outname, ".o");
-}
-
 static void ieee_write_file(void)
 {
     const struct tm * const thetime = &official_compile_time.local;
@@ -1507,20 +1498,22 @@ static const struct dfmt * const ladsoft_debug_arr[3] = {
 const struct ofmt of_ieee = {
     "IEEE-695 (LADsoft variant) object file format",
     "ieee",
+    ".o",
     OFMT_TEXT,
     32,
     ladsoft_debug_arr,
     &ladsoft_debug_form,
     NULL,
     ieee_init,
+    null_reset,
     nasm_do_legacy_output,
     ieee_out,
     ieee_deflabel,
     ieee_segment,
+    NULL,
     ieee_sectalign,
     ieee_segbase,
     ieee_directive,
-    ieee_filename,
     ieee_cleanup,
     NULL                        /* pragma list */
 };
